# include <stdio.h>
# include <stdlib.h>
# include "mex.h"        /* the algorithm is connect to matlab */
# include "math.h"
# define PI 3.1415926
# define ABS(x) ((x)>0? (x):(-(x)))
# define MAX(x, y) ((x)>(y)? (x):(y))
# define MIN(x, y) ((x)<(y)? (x):(y))

 /* variables */
 int n;                      /* number of images */
 int N;                      /* number of orientations */
 double **sI;                /* source images */
 double **rI;                /* the resulting transformed (geometry) images */
 double* sx, *sy;                 /* sizes of image */ 
 double *deltaX, *deltaY, *deltaOri;
 int tx; /* template size */

/* Interpolation: Nearest Neighbor */
/*
	(cx,cy): pivot of rotation
	(x,y,iOri): point to be transformed
    x: row; y: column
	(deltaX,deltaY,deltaOri): the transform
	*xyo[3]: point after transformation
*/

/* New implementation: positive direction of x is now downward */
void geometricTransform(int cx,int cy,int x,int y,int iOri,int deltaX,int deltaY,int deltaOri,int *xyo)
{
	double alpha,radius;
	/* First, rotate */
	xyo[2] = iOri+deltaOri;
	while(xyo[2]<0)
		xyo[2] += N;	/*If we use other filters than Gabor, we may need to module by 2N.*/
	if(xyo[2]>=N)
		xyo[2] = xyo[2] % N;
	alpha = atan2(cx-x,y-cy) + deltaOri*PI/N;
	radius = sqrt((y-cy)*(y-cy)+(x-cx)*(x-cx));
	/* Then, shift the center */
	cx += deltaX; cy += deltaY;
	xyo[0] = floor(0.5 + cx - radius*sin(alpha));
	xyo[1] = floor(0.5 + cy + radius*cos(alpha));
}


void rotateMap()
{
	int i,x,y;
	int xyo[3];
   
    
	for(i=0;i<n;i++) /* fill in each pixel of the destination image */
		for(x=0;x<tx;x++)
			for(y=0;y<tx;y++)
			{
                
				geometricTransform(floor(tx/2),floor(tx/2),x,y,0,deltaX[i],deltaY[i],deltaOri[i],xyo);
				if(xyo[0]>=sx[i] || xyo[0]<0 || xyo[1]>=sy[i] || xyo[1]<0)
					continue;
				else
					rI[i][xyo[0]+xyo[1]*(int)(sx[i])] = sI[i][y*tx+x];
			}
}


/* mex function is used to pass on the pointers and scalars from matlab, 
   so that heavy computation can be done by C, which puts the results into 
   some of the pointers. After that, matlab can then use these results. 
   
   So matlab is very much like a managing platform for organizing the 
   experiments, and mex C is like a work enginee for fast computation. */

void mexFunction(int nlhs, mxArray *plhs[], 
                 int nrhs, const mxArray *prhs[])                
{
 int i, j, c; 
 mxArray *f;  
 
 c = 0; /* counter for input pointers and scalars */
 n = floor(mxGetScalar(prhs[c++])+.5);  /* number of images */
 N = floor(mxGetScalar(prhs[c++])+.5);  /* number of orientations */
 sI = mxCalloc(n, sizeof(double*));   /* fitered images */
 for (i=0; i<n; i++)
 { 
       f = mxGetCell(prhs[c], i); 
       sI[i] = mxGetPr(f);    /* get pointers to filtered images */       
 }
 c++;
 rI = mxCalloc(n, sizeof(double*));   /* fitered images */
 for (i=0; i<n; i++)
 {
       f = mxGetCell(prhs[c], i); 
       rI[i] = mxGetPr(f);    /* get pointers to filtered images */   
 }
 c++;
 
 tx = floor(mxGetScalar(prhs[c++])+.5); /* template size */
 sx = mxGetPr(prhs[c++]);
 sy = mxGetPr(prhs[c++]);   /* size of images */
 deltaX = mxGetPr(prhs[c++]);
 deltaY = mxGetPr(prhs[c++]);
 deltaOri = mxGetPr(prhs[c++]);

 rotateMap();
}

     



 

                    