/***********************************************************************************


Input:  template and neighborhood within which the template is allowed to move.
Output: Rotation, shift, and score of the template matched on each patch.
		Due to space complexity, local transformations of individual Gabor elements are not returned.
 *      It is suggested that the user runs matching again to get the gabor deformations.

************************************************************************************/

# include <stdio.h>
# include <stdlib.h>
# include "mex.h"        /* the algorithm is connect to matlab */
# include "math.h"
# define PI 3.1415926
# define ABS(x) ((x)>0? (x):(-(x)))
# define MAX(x, y) ((x)>(y)? (x):(y))
# define MIN(x, y) ((x)<(y)? (x):(y))

double *double_vector(int n)
{
    double *v;
    v = (double*) mxCalloc (n, sizeof(double));
    return v; 
}

int *int_vector(int n)
{
    int *v; 
    v = (int*) mxCalloc (n, sizeof(int));
    return v;
}

double **double_matrix(int m, int n)
{
    double **mat; 
    int i; 
    mat = (double**) mxCalloc(m, sizeof(double*));
    for (i=0; i<m; i++)
        mat[i] = double_vector(n);
    return mat;
}

int **int_matrix(int m, int n)
{
    int **mat;
    int i;
    mat = (int**) mxCalloc(m, sizeof(int*));
    for (i=0; i<m; i++)
        mat[i] = int_vector(n);
    return mat;
}

/* getting the index of matlab image, (x,y) location, (sx, sy) sizes */
int px(int x, int y, int bx, int by)
{
   return (x + (y-1)*bx - 1);
}

int n;                      /* number of images */
int N;                      /* number of orientations */
double **fI;                /* filtered images */
int h;                      /* halfsizes of filters */
int bx, by;                 /* sizes of image */
int X0, Y0;				 /* position of top-left corner of block */
double *Mi, *Mx, *My, *Mm0, *Mm1;  /* storing selected bases */
double *sym;                 /* symbol of selected Gabors */
int L, ore;                 /* allowed ranges of shifting in location and orientation */
int sub;                    /* subsampling */
double *tss;                /* log-likelihood score for each testing image */
int Totalsketch;            /* total number of bases */
int **sinsh, **cossh;       /* store the shift to avoid repeated sin and cosin computation */
double *Sx1, *Sy1;
double *NB;
int sizeNB;

/* store the shift values so that we do not need to repeat the sin and cos computation */ 
void storeshift()
{
    int ind, l;
    double theta; 
    
    sinsh = int_matrix(N, L+L+1); 
    cossh = int_matrix(N, L+L+1); 
    for (ind=0; ind<N; ind++)        
    {
        theta = PI*ind/N; 
        for (l=-L; l<=L; l++)
         {
            sinsh[ind][l+L] = floor(l*sub*sin(theta)+.5); 
            cossh[ind][l+L] = floor(l*sub*cos(theta)+.5); 
        }   
    }
}
    
/* for Gabor(x, y, orientation = ind), find local maximum in image i */
double shiftmax(int i, int ind, int x, int y, int *rm) 
{
   double m;
   int x1, y1, l, ml, mx, my, de, d, d1, here, mo, md; 
 
   m = 0.;
   if ( L == 0 && ore == 0 )
   {
       /*printf("x=%d,Sx1=%d,y=%d,Sy1=%",x,Sx1[i],y,Sy1[i]); */
       if ((x>=1)&&(x<=Sx1[i])&&(y>=1)&&(y<=Sy1[i]))
       {
           here = px(x, y, Sx1[i], Sy1[i]);
           /*printf("x=%d,y=%d,m=%f",x,y,fI[ind*n+i][here]); */
           return fI[ind*n+i][here];
       }
       else
       {
           return m;
       }
   }

   for (l=-L; l<=L; l++)   
   {
        x1 = x + cossh[ind][l+L]; 
        y1 = y + sinsh[ind][l+L];   /* shifting in normal direction */
        if ((x1>=1)&&(x1<=Sx1[i])&&(y1>=1)&&(y1<=Sy1[i]))
        {
        here = px(x1, y1, Sx1[i], Sy1[i]);
        for (de=-ore; de<=ore; de++)
            {
              d = de+ind;
              d1 = d;
              if (d<0)
                 d1 = d+N;
              if (d>=N)
                  d1 = d-N;   /* shifting in orientation */
              if (fI[d1*n+i][here]>m)
                {
                   m = fI[d1*n+i][here];   /* local maximization */
                   ml = l; mx = x1; my = y1; mo = d1; md = de; 
                }
             }
        }
     }
   rm[0] = ml; rm[1] = mx; rm[2] = my; rm[3] = mo; rm[4] = md; 
   return(m);
}

/*
	(cx,cy): pivot of rotation
	(x,y,iOri): point to be transformed
    x: row; y: column
	(deltaX,deltaY,deltaOri): the transform
	*xyo[3]: point after transformation
*/
/* New implementation: positive direction of x is now downward */
void geometricTransform(int cx,int cy,int x,int y,int iOri,int deltaX,int deltaY,int deltaOri,int *xyo)
{
	double alpha,radius;
	/* First, rotate */
	xyo[2] = iOri+deltaOri;
	while(xyo[2]<0)
		xyo[2] += N;	/* If we use other filters than Gabor, we may need to module by 2N. */
	if(xyo[2]>=N)
		xyo[2] = xyo[2] % N;
	alpha = atan2(cx-x,y-cy) + deltaOri*PI/N;
	radius = sqrt((y-cy)*(y-cy)+(x-cx)*(x-cx));
	/* Then, shift the center */
	cx += deltaX; cy += deltaY;
	xyo[0] = floor(0.5 + cx - radius*sin(alpha));
	xyo[1] = floor(0.5 + cy + radius*cos(alpha));
}


void Cmp()
{
     int i, t, rm[5], xyo[3], k, pivotx, pivoty;
     double mm, *tmp;
     int j, templateDeltaO, templateDeltaX, templateDeltaY; 
     double softmax;
     j=0;
/*      tmp = double_vector(n*sizeNB);  scores for each image, each transformation (it is equivalent to a local saliency map)*/
	 pivotx = X0+floor(bx/2); /* These two lines should be moved inside the loop, if image sizes are different. */
	 pivoty = Y0+floor(by/2);
     
     /*printf("anchor0\n"); */
     
	 for (i=0; i<n; i++) /* for each image */
	 {
		for(k=0; k<sizeNB; k++) /* For each point (x,y,o) within neighborhood, transform the template to this point. */
		{
			templateDeltaX = NB[k];
			templateDeltaY = NB[sizeNB+k];
			templateDeltaO = NB[2*sizeNB+k];
            /*printf("delta xyo=(%d,%d,%d)\n",templateDeltaX,templateDeltaY,templateDeltaO); */
			for(t=0;t<Totalsketch;t++)
			{
                /*printf("a");*/
				geometricTransform(pivotx,pivoty,X0+Mx[t],Y0+My[t],Mi[t],templateDeltaX,templateDeltaY,templateDeltaO,xyo);
                /*printf("b");*/
                
                
				mm = shiftmax(i, xyo[2], xyo[0], xyo[1], rm);
                
                /*printf("c");*/
				/* tmp[k*n+i] += Mm0[t]*mm - Mm1[t]; */
                tss[k*n+i] += Mm0[t]*mm - Mm1[t];
                /*printf("d");*/
                
                if( mm < 0 )
                {
                    printf("xyo=(%d,%d,%d)\n",xyo[0],xyo[1],xyo[2]);
                    printf("mm=%.3f\n",mm);
                    printf("Mm0[t]=%.3f\n",Mm0[t]);
                    printf("Mm1[t]=%.3f\n",Mm1[t]);
                    printf("tss[k*n+i]=%.3f\n",Mm0[t]*mm - Mm1[t]);

                }
			}

		}
	 }
     
     

}


/* mex function is used to pass on the pointers and scalars from matlab, 
   so that heavy computation can be done by C, which puts the results into 
   some of the pointers. After that, matlab can then use these results. 
   
   So matlab is very much like a managing platform for organizing the 
   experiments, and mex C is like a work enginee for fast computation. */

void mexFunction(int nlhs, mxArray *plhs[], 
                 int nrhs, const mxArray *prhs[])
{
 int ind, i, j, c; 
 mxArray *f;  

 c = 0; /* counter for input pointers and scalars */
 n = floor(mxGetScalar(prhs[c++])+.5);  /* number of images */

 N = floor(mxGetScalar(prhs[c++])+.5);  /* number of orientations */

 fI = mxCalloc(n*N, sizeof(double*));   /* fitered images */
 for (i=0; i<n; i++)
 {
     for (ind=0; ind<N; ind++)
      {  
       f = mxGetCell(prhs[c], ind*n+i); 
       fI[ind*n+i] = mxGetPr(f);    /* get pointers to filtered images */       
      }
    }
 c++;


 h = floor(mxGetScalar(prhs[c++])+.5);    /* half size of filters */
 
 bx = floor(mxGetScalar(prhs[c++])+.5);
 by = floor(mxGetScalar(prhs[c++])+.5);   /* size of template */
 X0 = floor(mxGetScalar(prhs[c++])+.5)-1;
 Y0 = floor(mxGetScalar(prhs[c++])+.5)-1;
 Sx1 = mxGetPr(prhs[c++]);
 Sy1 = mxGetPr(prhs[c++]);				  /* size of images */
     
 Mi = mxGetPr(prhs[c++]);   /* orientation of selected Gabor */
 Mx = mxGetPr(prhs[c++]);
 My = mxGetPr(prhs[c++]);   /* (x, y) position of selected Gabor */
 Mm0 = mxGetPr(prhs[c++]);   /* prior given parent = background */
 Mm1 = mxGetPr(prhs[c++]);  /* prior given parent = foregroud */
 
 L = floor(mxGetScalar(prhs[c++])+.5);     /* range of shifting along normal direction of Gabor */
 ore = floor(mxGetScalar(prhs[c++])+.5);   /* range of shifting in orientation */
 sub = floor(mxGetScalar(prhs[c++])+.5);   /* sub-sampling of Gabor filters to improve speed */
 Totalsketch = floor(mxGetScalar(prhs[c++]));  /* total number of bases */
 tss = mxGetPr(prhs[c++]);                        /* map of log-likelihood score */
 
 NB = mxGetPr(prhs[c]);
 
 sizeNB = mxGetM(prhs[c++]);  /* number of points inside NB */

 storeshift();
 Cmp();
 /*printf("after cmp()\n");*/
}