function [S1Maps voidMaps voidMap2] = getS1Maps(allFilter,images,XYS,BB_size_per_scale)
%
% compute S1 Map from gray-level images
% <tx> : template size (it is a square)
%

% As specified in getLocs.m, there are 7 scales in total.
nScale = length(BB_size_per_scale);
norient = length(allFilter);
nLoc = size(XYS,1);
void_thres = 0.05;
h = (size(allFilter{1}, 1)-1)/2;  % half size of Gabor
localHalfx = floor(h*2); localHalfy = localHalfx;
tx = BB_size_per_scale(1); % the window size

nImage = length(images);
S1Maps = cell(nImage,nScale,norient);
voidMaps = cell(nImage,1);
for iImage = 1:nImage
    im = images{iImage};
    % scale = 0
    [sx sy] = size(im);
    for iScale = 1:nScale
%         disp(iScale);
        resizedBBsize = BB_size_per_scale(iScale);
        if sx >= sy
            newSx = resizedBBsize;
            newSy = floor(.5+resizedBBsize*sy/sx); 
        else
            newSy = resizedBBsize;
            newSx = floor(.5+resizedBBsize*sx/sy);
        end
        tmp = ApplyFilterfftSame({imresize(im,[newSx,newSy],'nearest')}, allFilter);
        % shift the S1 Map to the center:
        % and compute the voidMaps
        for o = 1:norient
            S1Maps{iImage,iScale,o} = zeros(resizedBBsize,resizedBBsize);
            if sx >= sy
                S1Maps{iImage,iScale,o}(:,(1:newSy)+floor((resizedBBsize-newSy)/2)) = tmp{o}(:,:);
            else
                S1Maps{iImage,iScale,o}((1:newSx)+floor((resizedBBsize-newSx)/2),:) = tmp{o}(:,:);
            end
        end
    end
    
    voidMaps{iImage} = zeros(nLoc,1);
    
    for iScale = 1:nScale
        II = cell(norient,1);
        aveIIoverOri{iScale} = zeros(size(S1Maps{iImage,iScale,1}));
        for o = 1:norient
            II{o} = cumsum(cumsum(S1Maps{iImage,iScale,o}),2);
            aveIIoverOri{iScale} = aveIIoverOri{iScale} + II{o} / norient;
        end
    end
    
    maxAve = zeros(nScale,1);
    ave = zeros(nLoc,1);
    for iLoc = 1:nLoc
        xx = XYS(iLoc,1); yy = XYS(iLoc,2); ss = XYS(iLoc,3);
        
        % check if the window falls outside of valid region
        
%         resizedBBsize = BB_size_per_scale(ss);
        
%         relax = tx/4;
%         
%         if sx >= sy
%             newSy = floor(.5+resizedBBsize*sy/sx);
%             bInside = yy+relax >= (resizedBBsize - newSy)/2 && yy-relax <= (resizedBBsize + newSy)/2 - tx ;
%         else
%             newSx = floor(.5+resizedBBsize*sx/sy);
%             bInside = xx+relax >= (resizedBBsize - newSx)/2 && xx-relax <= (resizedBBsize + newSx)/2 - tx ;
%         end
%         
%         voidMaps{iImage}(iLoc) = ~bInside;
%         if ~bInside
%             continue;
%         end
        
        %
        if xx > 1 && yy > 1
            ave(iLoc) = aveIIoverOri{ss}(xx+tx-1,yy+tx-1) - aveIIoverOri{ss}(xx-1,yy+tx-1) ...
                - aveIIoverOri{ss}(xx+tx-1,yy-1) + aveIIoverOri{ss}(xx-1,yy-1);
        end
        if xx == 1 && yy > 1
            ave(iLoc) = aveIIoverOri{ss}(xx+tx-1,yy+tx-1) ...
                - aveIIoverOri{ss}(xx+tx-1,yy-1) ;
        end
        if xx > 1 && yy == 1  
            ave(iLoc) = aveIIoverOri{ss}(xx+tx-1,yy+tx-1) - aveIIoverOri{ss}(xx-1,yy+tx-1);
        end
        if xx == 1 && yy == 1  
            ave(iLoc) = aveIIoverOri{ss}(xx+tx-1,yy+tx-1);
        end
        ave(iLoc) = ave(iLoc) / tx / tx;
        
        if ave(iLoc) > maxAve(ss)
            maxAve(ss) = ave(iLoc);
        end
        
    end
    
    for iLoc = 1:nLoc
        voidMaps{iImage}(iLoc) = (ave(iLoc) < void_thres);
    end
    
    for iScale = 1:nScale
        resizedBBsize = BB_size_per_scale(iScale);
        ClocalNormalizeDouble(resizedBBsize, resizedBBsize, norient, h, localHalfx, localHalfy, S1Maps(iImage,iScale,:)); 
    end

end


%% sigmoid transform
load nlf Upperbound
for iImage = 1:nImage
    Ctransform(nScale, norient, S1Maps(iImage,:,:), BB_size_per_scale, BB_size_per_scale, Upperbound);    
end

%% obtain void maps
voidMap2 = cell(nImage,nScale);
for iImage = 1:nImage
    for s = 1:nScale
        resizedBBsize = BB_size_per_scale(s);
        voidMap2{iImage,s} = zeros(resizedBBsize, resizedBBsize);
    end

    for j = 1:nLoc
        top = XYS(j,1);
        left = XYS(j,2);
        bottom = top + tx-1;
        right = left + tx-1;
        if voidMaps{iImage}(j) > 0
            voidMap2{iImage,XYS(j,3)}(top:bottom,left:right) = 1;
        end
    end
end

a = 1;
