clear all
close all

% the first level of word learning
% serves as initialization for finding parts guided by parent

% reminder:
% use getLocs for seed proposals
% use getFineLocs for detection

%% load natural images data
tx = 80; % the window size
if ~exist(sprintf('NegS1Maps_%d.mat',tx),'file')
    dumpNeg(tx);
end
load(sprintf('NegS1Maps_%d.mat',tx));
negData = struct('S1Maps',{S1Maps},'voidMaps',{voidMaps},'XYS',XYS,'BB_size_per_scale',BB_size_per_scale);
clear S1Maps; clear voidMaps; clear XYS; clear BB_size_per_scale;

%% read images
mkdir result
Iname = dir('pos/*.jpg');
nImage = length(Iname);
images = cell(length(Iname),1);
for iImage = 1:length(images)
    im = imread(['pos/' Iname(iImage).name]);
    if size(im,3) == 3
        im = rgb2gray(im);
    end
    images{iImage} = double(im);
end


score_thres_preset = 5;

%% compute multi-scale S1 Maps (locally normalized) and the void maps (to shut down structure-less patches)
[XYS BB_size_per_scale grid_step_size] = getFineLocs(tx,5); % pre-compute the more densely sampled localizations (all locations, scales, orientations) 
nLoc = size(XYS,1);

norient = 16;
load('nlf.mat','scale','Upperbound');
[allFilter allsymbol] = makefilter(scale, norient);  % generate Gabor filters 

if ~exist(sprintf('S1Maps_saved_%d.mat',tx),'file')
    [S1Maps voidMaps] = getS1Maps(allFilter,images,XYS,BB_size_per_scale);
    save(sprintf('S1Maps_saved_%d.mat',tx),'S1Maps','voidMaps');
else
    load(sprintf('S1Maps_saved_%d.mat',tx),'S1Maps','voidMaps');
end

coverMaps = cell(nImage,1);
for i = 1:nImage
    coverMaps{i} = zeros(nLoc,1);
end

%% preparation for the EM learning
oriRange = [0 2 4 6 8 10 12 14 16 18 20 22 24 28 30]; % rotated angles of the template 
K = 20; % # maximum number of nearest neighbors in E step
Lrange = 3;
Orange = 1;
sub = 1;
maxNumWord = 1;
Totalsketch = 25;
initialLocs = cell(maxNumWord,1);
templates = cell(maxNumWord,1);
extents = cell(maxNumWord,1);
highestScores = cell(maxNumWord,1);

%% EM and Local learning

%
% We start from the 1st localization (i.e. the whole image) of the 4th image.
%
seedImage = 4; 
seedLoc = 1;

    % start from this single patch of the first image

    % keep track of the K NN
    extents{seedLoc} = zeros(K,6); % ( x, y, scale, orientation, image ID, iLoc )
    lowScore = -1000;
    highestScores{seedLoc} = ones(K,1) * lowScore;
    
    %% initial M step
    ss = XYS(seedLoc,3); xx = XYS(seedLoc,1); yy = XYS(seedLoc,2);
    collectedS1Maps = cell(1,norient);
    for o = 1:norient
        collectedS1Maps{1,o} = S1Maps{seedImage,ss,o}(xx:xx+tx-1,yy:yy+tx-1);
    end
    dataWeight = 1;
    Lrange0 = 0; Orange0 = 0;
    learnedModel = ABlearn(negData,dataWeight,collectedS1Maps,allsymbol,allFilter,Lrange0,Orange0,sub,Totalsketch);
    
    iter = 1;
    maxNumEMSteps = 7;
    
    %% the succeeding EM steps
    while iter < maxNumEMSteps
        
        lowScore = -1000;
        highestScores{seedLoc} = ones(K,1) * lowScore;
        extents{seedLoc} = zeros(K,6); % ( x, y, scale, orientation, image ID, iLoc )
        
        %% E step
        
        % Select one best instance from each example
        storedFullScores = cell(nImage,1);
        
        for jImage = 1:nImage
            % scan everywhere
            [scores bestOriPerLoc storedFullScores{jImage}] = ABtest_one(XYS,oriRange,learnedModel,S1Maps(jImage,:,:));
            scores(coverMaps{jImage}>0) = lowScore;
            candidateLocIndice = find(scores>min(highestScores{seedLoc})); % find those locs with competitive scores
            if numel(candidateLocIndice) == 0
                continue;
            end

            % !! keep only one top match per image
            [maxScore idx] = max(scores);
            for iCandLoc = idx
                [val ind] = min(highestScores{seedLoc});
                % replace the lowest score among the K highest;
                % replace the corresponding extent
                if scores(iCandLoc) > val
                    % local inhibition:
                    % If it overlaps with previously selected extent, then do nothing.
                    % We define the local neighborhood below.
                    current_x = XYS(iCandLoc,1);
                    current_y = XYS(iCandLoc,2);
                    current_s = XYS(iCandLoc,3);
                    
                    neighborhood = [];
                    for deltaX = -grid_step_size*2 : grid_step_size : grid_step_size*2
                        for deltaY = -grid_step_size*2 : grid_step_size : grid_step_size*2
                            neighborhood = [neighborhood;...
                                current_x + deltaX, current_y + deltaY, current_s, bestOriPerLoc(iCandLoc), jImage ];
                        end
                    end
                    
                    A = intersect( extents{seedLoc}(:,1:5), ...
                        neighborhood...
                        , 'rows');
                    if numel(A) > 0
                        continue;
                    end

                    highestScores{seedLoc}(ind) = scores(iCandLoc);
                    extents{seedLoc}(ind,:) = [XYS(iCandLoc,1) XYS(iCandLoc,2) XYS(iCandLoc,3) bestOriPerLoc(iCandLoc) jImage iCandLoc];
                else
                    break;
                end
            end
        end
        
        % sort the K instances
        [sorted IDX] = sort(highestScores{seedLoc}(:),'descend');
        highestScores{seedLoc} = sorted;
        extents{seedLoc} = extents{seedLoc}(IDX,:);
        
        if sum(highestScores{seedLoc}>lowScore+1) == 0
            break;
        end
        
        % Threshold for the log-likelihood score.
        % Only examples with higher scores can be selected as training examples in the next M step. 
        score_thres = min(score_thres_preset,max(-1e-3+highestScores{seedLoc}));
        
        nValid = sum(highestScores{seedLoc}>score_thres); % number of valid training examples 
        
        if nValid == 0 % terminate if no good examples are found
            disp('no good examples are found');
            break;
        end
        
        %% display
        % Now crop the k-NN instances for display purpose
        destImages = cell(nValid,1);
        for i = 1:nValid
            destImages{i} = zeros(tx,tx);
        end
        srcImages = cell(nValid,1); sxList = zeros(nValid,1); syList = zeros(nValid,1);
        srcImages_overlay = cell(nValid,1);
        BB_per_image = zeros(nValid,1);
        for i = 1:nValid
            imageID = extents{seedLoc}(i,5);
            if imageID == 0
                continue;
            end
            imageScale = extents{seedLoc}(i,3);
            [originalSX originalSY] = size(images{imageID});

            % fit the original image tightly into this bounding box
            resizedBBsize = BB_size_per_scale(imageScale);
            
            srcImages{i} = zeros(resizedBBsize,resizedBBsize);
            if originalSX >= originalSY
                sxList(i) = resizedBBsize;
                syList(i) = floor(.5+resizedBBsize*originalSY/originalSX);
                srcImages{i}(:,(1:syList(i))+floor((resizedBBsize-syList(i))/2)) = imresize(images{imageID},[sxList(i) syList(i)],'nearest');
            else
                syList(i) = resizedBBsize;
                sxList(i) = floor(.5+resizedBBsize*originalSX/originalSY);
                srcImages{i}((1:sxList(i))+floor((resizedBBsize-sxList(i))/2),:) = imresize(images{imageID},[sxList(i) syList(i)],'nearest');
            end
            
             BB_per_image(i) = resizedBBsize;
             
             rotatedTemplate = zeros(resizedBBsize,resizedBBsize);
             rotatedTemplate(extents{seedLoc}(i,1)-1+(1:tx), extents{seedLoc}(i,2)-1+(1:tx) ) =...
                 imrotate(learnedModel.sym,extents{seedLoc}(i,4)*180/norient,'crop');
             storedRotatedTemplate{i} = rotatedTemplate;
             mask = zeros(tx,tx); mask([8:10 end-10:end-8],8:end-8) = 1; mask(8:end-8,[8:10 end-10:end-8]) = 1;
             rotatedMask = zeros(resizedBBsize,resizedBBsize);
             rotatedMask(extents{seedLoc}(i,1)-1+(1:tx), extents{seedLoc}(i,2)-1+(1:tx) ) =...
                 imrotate(mask,extents{seedLoc}(i,4)*180/norient,'crop');
             srcImages_overlay{i} = srcImages{i} + rotatedMask * 150;
        end

        CCropInstance(nValid,norient,srcImages,destImages,tx,BB_per_image,BB_per_image,extents{seedLoc}(1:nValid,1),extents{seedLoc}(1:nValid,2),extents{seedLoc}(1:nValid,4));
 
        % display the current template
        % display the current K-NN instances
        
        imageScale = XYS(seedLoc,3);
        [originalSX originalSY] = size(images{seedImage});
        resizedBBsize = BB_size_per_scale(imageScale);
 
        overlayedOnExamples_vec = displayImagesAndSketches(srcImages_overlay,storedRotatedTemplate,2,120,120);

        towrite = -learnedModel.sym;
        towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
        imwrite(towrite,sprintf('template_iter%d.png',iter) );

        imwrite(overlayedOnExamples_vec,sprintf('matched_iter%d.png',iter) );
        disp(highestScores{seedLoc});
        
        if max(highestScores{seedLoc}) < -999
            disp('fail to find any top instances !!');
            pause;
        end
        
        %% M step
        if nValid <= 1
            break;
        end
        
        % collect the S1 maps
        collectedS1Maps = cell(nValid,norient);
        for i = 1:nValid
            for o = 1:norient
                collectedS1Maps{i,o} = zeros(tx,tx);
            end
        end
        x0List = zeros(nValid,1)+1; y0List = zeros(nValid,1)+1;

        wholeMapsForGeoTransform = cell(nValid,norient);
        Sx1 = zeros(nValid,1);
        Sy1 = zeros(nValid,1);
        for i = 1:nValid
            imageID = extents{seedLoc}(i,5);
            imageScale = extents{seedLoc}(i,3);
            wholeMapsForGeoTransform(i,:) = S1Maps(imageID, imageScale, :);
            Sx1(i) = BB_size_per_scale(imageScale,1);
            Sy1(i) = BB_size_per_scale(imageScale,1);
        end
        
        % The actual computation of cropping and rotating S1 patches
        % About this function: it does not reversely rotate the original S1 map directly,  
        %   instead, it rotate the empty resulting map under the same angle as the rotated template 
        %   and fetch Gabor responses.
        geoTransform(nValid,norient,wholeMapsForGeoTransform,collectedS1Maps,...
            tx,Sx1,Sy1,extents{seedLoc}(:,1)-1,extents{seedLoc}(:,2)-1,extents{seedLoc}(:,4));
        
        % update data weights
        dataWeight = 1-1/(1+exp(highestScores{seedLoc}(1:nValid)-score_thres));
        dataWeight = dataWeight / sum(dataWeight);
        
        % If there is only one example in the training set, then do not apply local maximization in learning 
        if nValid == 1
            Lrange_used = 0; Orange_used = 0;
        else
            Lrange_used = 3; Orange_used = 1;
        end
        
        collectedS1Maps = collectedS1Maps(highestScores{seedLoc}>score_thres,:);
        
        % The actual computation in learning
        learnedModel = ABlearn(negData,dataWeight,collectedS1Maps(highestScores{seedLoc}>score_thres,:),allsymbol,allFilter,Lrange_used,Orange_used,sub,Totalsketch);
        
        %% increment the iteration counter
        iter = iter + 1;
    end

    % save result
    topScores = highestScores{seedLoc}; topExtents = extents{seedLoc};
    save(sprintf('result/%d.mat',seedLoc),'topScores','learnedModel','storedRotatedTemplate','srcImages','topExtents','storedFullScores');
